#include "main.h"

uint8_t 			serbuffer[SERBUFSIZE];
uint16_t			serRdPointer;
volatile uint16_t	serInPointer;
volatile int16_t	buf_usage;

ISR (USART0_RX_vect)
{
uint8_t c;
	c = UDR0;			// get one more char from serial line
	serbuffer[serInPointer++] = c;
	serInPointer &= (SERBUFSIZE - 1);
	buf_usage++;
}	// end of ISR

int main(void)
{
uint8_t	c;
	initPorts();
	initUART();
	sei();	// allow interrupts
	while (1)
	{
		c = uart_getChar();
		uart_putChar(c);
		if (c == '\r') uart_putChar('\n');	// expand CR to CRLF
	}
} // end of main

void initUART(void)
{
	// Set baud rate
	UBRR0H = (uint8_t) (USART_UBRR_VALUE>>8);
	UBRR0L = (uint8_t) USART_UBRR_VALUE;
	// Enable rec interrupt, receiver and transmitter
	UCSR0B = (1<<RXCIE0) | (1<<RXEN0) | (1<<TXEN0);
	UCSR0C = (1<<UCSZ01) | (1<<UCSZ00);	// 8N1 format
}	// end of initUART

void initPorts(void)
{
	DDRA = ALLPINSOUT;	// port A is Output: A0..A7
	DDRB = ALLPINSOUT;	// port B is Output: A8..A15
	DDRC = ALLPINSOUT;  // port C is Output: D0..D7
	/* contents of port D must be set before data direction,
	   otherwise Byte 0 of the RAM might get deleted!  */
	PORTD = PORTD | 0b11111111;  // put ram inactive (D6 = /WE, D7=/OE)
	DDRD = ALLPINSIN | 0b11100000;	/* port D7,D6,D5 are output 
	        D5 is /RESET for Z180
			D6 is /WE for SRAM
			D7 is /OE for SRAM
			*/	
	PORTA = 0;
	PORTB = 0;	// set Addr to 0
	PORTC = 0;	// set Data to 0
	PORTD = PORTD & ~RESET;  // put Z180 in reset state (D5 = /RESET)
}	// end of initPorts

void uart_putChar(uint8_t c)
{
	while (!(UCSR0A & (1<<UDRE0)))  // wait until send allowed
	{ 

	}
	UDR0 = c;
}	// end of uart_putChar

uint8_t uart_getChar(void)
{
uint8_t	c;
	while (buf_usage <= 0)
	{ 

	}
	cli();
	c = serbuffer[serRdPointer++];
	serRdPointer &= (SERBUFSIZE - 1);
	buf_usage--;
	sei();
	return c;
}	// end of uart_getChar
